<label id="primary-label" for="primary-checkbox">≡</label>
<input id="primary-checkbox" type="checkbox" checked>
<nav id="primary" class="sidebar">
  <header>
    <h1 class="title"><a href="" class="plain-link">$title</a></h1>
  </header>
  <!--
  <div class="dark-mode-switch">
    <input type="checkbox" id="js-dark-mode-switch">
  </div>
  -->
  <div class="search-box">
    <span>⌕</span>
    <input class="transparent search-input" type="search">
  </div>
  <div class="nav-items">
    <a href="#" class="nav-item plain-link">Home</a>
    <a href="#tags" class="nav-item plain-link">Tags</a>
    <a href="#references" class="nav-item plain-link">References</a>
  </div>
  <br>
  <br>
  <div class="nav-items">
    <a href="#review/" class="nav-item plain-link">Review</a>
  </div>
</nav>
<nav id="secondary" class="sidebar"></nav>
